# noinspection PyPep8Naming
def sampleFunction(x: int, y: float) -> float:
    """
    Multiply int and float sample.

    :param x: x value
    :type x: int
    :param y: y value
    :type y: float
    :return: result
    :return type: float
    """
    return x * y

def sampleFunction2(x2: int, y2: float) -> float:
    """
    Multiply int and float sample.

    :param x2: x value
    :type x2: int
    :param y2: y value
    :type y2: float
    :return: result
    :return type: float
    """
    return x2 * y2

