from cloudmesh.openapi.scikitlearn import SklearnGenerator

input_sklibrary = 'sklearn.linear_model.LogisticRegression'
model_tag = 'JagsLogis1'
SklearnGenerator.Sklearngenerator(input_sklibrary,model_tag)
